import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.RadialGradientPaint;
import java.awt.Graphics2D;

/**
 * Write a description of class AimDisplay here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class AimDisplay extends Actor
{
    private Worm player;
    
    private int radius;
    private int curX;
    private int curY;
    
    public static final double MIN_PROP = Worms_Land.MIN_FORCE / Worms_Land.MAX_FORCE;

    public AimDisplay(Worm player, int radius, boolean showTarget) {
        this.player = player;
        this.radius = radius;
        redraw(0, 0);
    }
    
    public void act() {
        if (getWorld().getObjects(Weapon_Menu.class).isEmpty() && getWorld().getObjects(Break_Menu.class).isEmpty()) {
            run();
        }
    }
    
    public void run()  {
        if (player != null) {
            setLocation(player.getX(), player.getY());
        }
        FullScreenMouseInfo info = FullScreenWindow.getMouseInfo();
        if (info != null)
        {
            int xDir = info.getX() - getX();
            int yDir = info.getY() - getY();
            
            if (yDir >= -1)
            {
                yDir = -1;
            }
            
            double angle = Math.atan2(yDir, xDir);
            
            if (angle >= -Math.toRadians(Worms_Land.LOWEST_ANGLE))
            {
                angle = -Math.toRadians(Worms_Land.LOWEST_ANGLE);
            }
            if (angle <= -Math.toRadians(180 - Worms_Land.LOWEST_ANGLE))
            {
                angle = -Math.toRadians(180 - Worms_Land.LOWEST_ANGLE);
            }
            
            double distProp = Math.hypot(xDir, yDir) / (double)radius;
            
            if (distProp > 1)
            {
                distProp = 1;
            }
            else if (distProp < MIN_PROP)
            {
                distProp = MIN_PROP;
            }
            
            
            
            double forceFactor = (distProp - MIN_PROP) / (1 - MIN_PROP);
            
            redraw(angle, forceFactor);
            
            if (FullScreenWindow.mouseClicked(null))
            {
                player.fire(angle, forceFactor);
                getWorld().removeObject(this);
            }
        }
    }    
    
    private void redraw(double radianAngle, double forceFactor) {
         float[] spacing = {0.0f, (float)(MIN_PROP - 0.001f), (float)MIN_PROP, 1.0f};
         Color[] colors = {new Color(0.0f, 0.0f, 0.0f, 0.0f), new Color(0.0f, 0.0f, 0.0f, 0.0f), new Color(1.0f, 0.0f, 0.0f, 0.1f), new Color(1.0f, 0.0f, 0.0f, 0.5f)};
         RadialGradientPaint p = new RadialGradientPaint(radius, radius, radius, spacing, colors);
         
         double dist = (MIN_PROP + (forceFactor * (1 - MIN_PROP))) * (double)radius;
         
         GreenfootImage img = new GreenfootImage(radius*2, radius*2);
         Graphics2D g = img.getAwtImage().createGraphics();
         g.setPaint(p);
         g.fillArc(0, 0, radius * 2, radius * 2, Worms_Land.LOWEST_ANGLE, 180 - 2 * Worms_Land.LOWEST_ANGLE);
         g.dispose();
         img.setColor(new Color(1.0f, 0.0f, 0.0f, 0.8f));
         curX = radius + (int)(dist * Math.cos(radianAngle));
         curY = radius + (int)(dist * Math.sin(radianAngle));
         img.drawLine(radius, radius, curX, curY);
         img.drawLine(curX - 5, curY - 5, curX + 5, curY + 5);
         img.drawLine(curX - 5, curY + 5, curX + 5, curY - 5);
         
         setImage(img);
    }
}
