import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.*;

/**
 * An explosion. It starts by expanding and then collapsing. 
 * The explosion will explode other obejcts that the explosion intersects.
 * 
 * @author Poul Henriksen
 * @version 1.0.1
 */
public class Explosion extends Weapon
{
    private GreenfootImage[] images;
    
    private double imageSize;
    
    private int size = 0;
    private int increment = 1;
    
    public Explosion(double size) {
        imageSize = size;
        initialiseImages();
        changeImage(images[0]);
        GreenfootSound explosion = new GreenfootSound("Explosion.mp3");
        explosion.setVolume(Settings.effectVolume);
        if (size < 3) {
            explosion.setVolume((int) (100.0*size/3.0));
        }
        explosion.play();
    }
    
    private void initialiseImages() {
        if (images == null) {
            GreenfootImage baseImage = new GreenfootImage("Explosion.png");
            int maxSize = (int) (baseImage.getWidth()*imageSize);
            int delta = maxSize/15;
            int size = 0;
            images = new GreenfootImage[14];
            for(int i=0; i < 14; i++) {
                size += delta;
                images[i] = new GreenfootImage(baseImage);
                images[i].scale(size, size);
            }
        }
    }
    
    /**
     * Call the run method to run the game if the game is executed.
     */
    public void act() {
        if (getWorld().getObjects(Break_Menu.class).isEmpty()) {
            run();
        }
    }
    
    public void run() {
        changeImage(images[size]);
        size += increment;
        if(size >= 14) {
            increment = -increment;
            size += increment;
        }
        if(size <= 0) {
            getWorld().removeObject(this);
        }
    }
}
