import java.awt.image.BufferedImage;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Cursor;
import java.awt.Toolkit;
import javax.swing.JPanel;

/**
 * This panel is used to draw the image of the world onto it.
 * 
 * @author Gevater_Tod4711
 * @version 1.0
 */
public class FullScreenBufferPanel extends JPanel
{
    private BufferedImage contentImage;
    
    /**
     * Create a new FullScreenBufferPanel.
     */
    public FullScreenBufferPanel() {
        
    }
    
    /**
     * Set the next drawn image of the panel.
     * 
     * @param img
     *      The next drawn image of the panel.
     */
    public void setImage(BufferedImage img) {
        this.contentImage = img;
    }
    
    /**
     * Draw the worlds image onto the fullscreen panel.
     * 
     * @param g
     *      The Graphics object to draw the image.
     */
    public void paint(Graphics g) {
        super.paint(g);
        g.drawImage(contentImage, 0, 0, null);
    }
    
    /**
     * Set the mouse cursor for this panel.
     * 
     * @param image
     *      The image of the new mouse cursor.
     * 
     * @param cursorPoint
     *      The click point of the new cursor.
     */
    public void setMouseCursor(BufferedImage image, Point cursorPoint) {
        setMouseCursor(Toolkit.getDefaultToolkit().createCustomCursor(image, cursorPoint, "cursor"));
    }
    /**
     * Set the mouse cursor for this panel.
     * 
     * @param cursor
     *      The new cursor.
     */
    public void setMouseCursor(Cursor cursor) {
        setCursor(cursor);
    }
}
